/**
 * @fileOverview
 * @name sas.js
 * @author William da Rosa Fröhlich
 */

/**
 * @typedef {Object} SASController~GeneralSection - General section parameters.
 * @property {String} state - [enabled|disabled] Enable/disable survivability.
 * @property {String} default_exten - Set default extension.
 * @property {String} sip_trunking - Set SIP Trunking.
 * @property {String[]} codecs - Codecs to be allowed.
 */

/**
 * @typedef {Object} SASController~ExtensionSection - Extensions section parameters.
 * @property {String} username - Username.
 * @property {String} name - Name of the user.
 * @property {String} secret - The user's password.
 */

/**
 * @class SASController
 * @param {Object} $http - AngularJS service for HTTP communication.
 * @param {Object} $log - AngularJS service for logging.
 * @property {SASController~GeneralSection} general - General section parameters.
 * @property {SASController~ExtensionSection[]} account - Extension section parameters.
 * @property {String} textfile - Plaintext of sip.conf file.
 */
angular.module('ag561app').controller('SASController', ['$http', '$log', '$location',
                                      'FileUploader',
                                      function($http, $log, $location, FileUploader) {
  var url_sas = '/api/config/sas';
  var ctrl = this;
  ctrl.valid_port = true;
  ctrl.option = 'none';
  ctrl.invalid_config = 0;

  /* FileUploader */
  ctrl.upload_filename = '';
  ctrl.uploader = new FileUploader({'url': '/api/upload'});
  ctrl.uploader.queueLimit = 1;
  ctrl.uploader.removeAfterUpload = true;

  /**
   * Clear alert message from web page, but do this only
   * if we aren't with some error. We just want to erase
   * the message of danger if all the variables are correct.
   *
   * @function clearAlertMessages
   * @memberof SASController
   */
  ctrl.clearAlertMessages = function() {
    if (!ctrl.invalid_config) {
      ctrl.alert_danger = false;
      ctrl.alert_success = false;
    }
  };

  /**
   * Get general and accounts section from sip.conf file.
   * @function get
   * @memberof SASController
   */
  ctrl.get = function() {
    $http({
      method: 'GET',
      url: url_sas
      }).then(function(response) {
        // First function handles success
        ctrl.sas = response.data;
        ctrl.rules = response.data.rules;
        ctrl.extenSort();
        ctrl.sas.default_exten = " " + response.data.default_exten + " ";
        ctrl.sas.sip_trunking = " " + response.data.sip_trunking + " ";
        ctrl.state = response.data.state == "enabled";
        ctrl.addCodecsOptions();
        ctrl.updateDefaultExten(ctrl.sas.default_exten);
        ctrl.updateSipTrunking(ctrl.sas.sip_trunking);
      }, function(response) {
        // Second function handles error
        ctrl.error = response.data;
      });
  };

  ctrl.get();

  /**
   * Save the new configuration.
   * If some error is happening then show in
   * web page.
   *
   * @function save
   * @memberof SASController
   */
  ctrl.save = function() {
    ctrl.loading = true;

    ctrl.sas.codecs = [];
    for (var j = 0; j < ctrl.codecs_models.lists.Enabled.length; j++) {
      ctrl.sas.codecs.push(ctrl.codecs_models.lists.Enabled[j].label);
    }

    var data = ctrl.sas;
    data.state = ctrl.state ? "enabled" : "disabled";
    if (!ctrl.sas.default_exten){
      data.default_exten = "9900"
    }else{
      data.default_exten = ctrl.sas.default_exten.substring(1, ctrl.sas.default_exten.length - 1);
    }

    if (!ctrl.sas.default_exten){
      data.sip_trunking = "trunk1"
    }else{
      data.sip_trunking = ctrl.sas.sip_trunking.substring(1, ctrl.sas.sip_trunking.length - 1);
    }
    data.rules = ctrl.rules;

    $http({
      method: 'PUT',
      url: url_sas,
      data: data,
      }).then(function(response) {
        // First function handles success
        ctrl.loading = false;
        ctrl.alert_success = true;
        ctrl.get();
      }, function(response) {
        // Second function handles error
        ctrl.loading = false;
        ctrl.alert_danger = true;
      });
  };


  /**
  * Delete a rule from the rules list
  *
  * @function extenDelete
  * @memberof SASController
  */
  ctrl.extenDelete = function(index) {
      ctrl.sas.extens.splice(index, 1);
      ctrl.checkInvalidConfig();
  };

  function findUserName(name) {
    var i;
    for (i = 0; i < ctrl.sas.extens.length; i++) {
      if (ctrl.sas.extens[i].name == name)
        return ctrl.sas.extens[i];
    }

    return false;
  };

  /**
   * Start a new object for create a new exten.
   * Initializes default values for the new exten.
   *
   * @function addUser
   * @memberof SASController
   */
  ctrl.addUser = function() {
    ctrl.clearAlertMessages();
    var i = 0;
    var exten_name = 9899;
    if (ctrl.sas.extens.length !== 0)
      exten_name = parseInt(ctrl.sas.extens[ctrl.sas.extens.length-1].name);

    // Check if exten name already exists
    do {
      exten_name = exten_name + 1;
      exten_name = exten_name.toString();
    } while (findUserName(exten_name));

    // Start default configuration
    var newexten = {'secret': '',
                    'name': exten_name,
                    'username': exten_name};

    newexten.view = !newexten.view;
    ctrl.sas.extens.push(newexten);
    ctrl.checkInvalidConfig();
    ctrl.edit_index = ctrl.sas.extens.indexOf(newexten);
    ctrl.updateDefaultExten(ctrl.sas.default_exten);
  };

  function updateDisabledCodecsList(disabled_list, codec_to_remove) {
    // disabled_list is a Arry with objects, then we need looking for codec in all positions.
    for (var j = 0; j < disabled_list.length; j++) {
      if (disabled_list[j].label == codec_to_remove) {
        disabled_list.splice(j, 1);
      }
    }
  };

  ctrl.addCodecsOptions = function() {
    ctrl.codecs_models = {selected: null,
                          lists: {"Disabled": [{label: "alaw"}, {label: "ulaw"},
                                               {label: "g729"}, {label: "g726"},
                                               {label: "g723"}],
                                  "Enabled": []
                                 }
                          }

    for (var i = 0; i < ctrl.sas.codecs.length; i++) {
      ctrl.codecs_models.lists.Enabled.push({label: ctrl.sas.codecs[i]});
      updateDisabledCodecsList(ctrl.codecs_models.lists.Disabled, ctrl.sas.codecs[i]);
    }
  };

  /**
   * Check if some configuration is invalid. If true, then disable the
   * button to 'save and apply' config and show the 'danger message'.
   *
   * @function __checkInvalidConfig
   * @memberof SASController
   */
  ctrl.checkInvalidConfig = function() {
    ctrl.invalid_config = false;

    for (var i = 0; i < ctrl.sas.extens.length; i++) {
      var exten = ctrl.sas.extens[i]
      if (exten.invalid_port ||exten.invalid_exten ||
          exten.invalid_username || exten.invalid_password)
        ctrl.invalid_config = true;
      break;
    }

    for (var i = 0; i < ctrl.rules.length; i++) {
      var rule = ctrl.rules[i]
      if (rule.invalid_outgoing_called_pattern ||
          rule.invalid_outgoing_callerid_pattern ||
          rule.invalid_rule_name, rule.invalid_answer_timeout ||
          rule.invalid_sour_dest || ctrl.invalid_config)
        ctrl.invalid_config = true;
      break;
    }

    ctrl.alert_danger = ctrl.invalid_config;
  };

  /**
   * Validade Username
   * Only accept numbers.
   *
   * @function validateName
   * @memberof SASController
   */
  ctrl.validateName = function(exten) {
    var regex = /^\d+$/;

    exten.invalid_exten = false;
    if (!regex.test(exten.name) || exten.name == "general") {
      exten.invalid_exten = true;
    } else {
      // Check if this name already exists in another exten
      var count = 0;
      for (var i = 0; i < ctrl.sas.extens.length; i++) {
        if (ctrl.sas.extens[i].name == exten.name)
          count++;
      }

      if (count > 1) {
        exten.invalid_exten = true;
      }
    }
    ctrl.checkInvalidConfig();
  };

  /**
   * Validade Username
   * Only accept numbers.
   *
   * @function validateUsername
   * @memberof SASController
   */
  ctrl.validateUsername = function(exten) {
    var regex = /^\d+$/;

    if (!regex.test(exten.username)) {
      // Check if username is valid
      exten.invalid_username = true;
    } else {
      exten.invalid_username = false;
    }

    ctrl.checkInvalidConfig();
  };

  /**
   * Validade Secret
   * It can not be a string with space, ",", ";".
   *
   * @function validatePassword
   * @memberof SASController
   */
  ctrl.validatePassword = function(exten) {
    var regex = /^[^\s;,!#]+$/;
    exten.invalid_password = true;

    if (exten.secret.length > 0 && regex.test(exten.secret)) {
      exten.invalid_password = false;
    } else if (exten.secret.length == 0){
      exten.invalid_password = false;
    }

    ctrl.checkInvalidConfig();
  };

  /**
   * Update automatically Default Extension.
   * @function updateDefaultExten
   * @memberof SASController
   */
  ctrl.updateDefaultExten = function() {
    if (!ctrl.sas.default_exten){
      if (!ctrl.sas.extens[0]){
        ctrl.invalid_called_exten = true;
      }else{
        ctrl.sas.default_exten = " " + ctrl.sas.extens[0].name + " ";
        ctrl.invalid_called_exten = false;
      }
    }else{
      ctrl.invalid_called_exten = false;
    }
  };


  /**
   * Update automatically SIP Trunking.
   * @function updateSipTrunking
   * @memberof SASController
   */
  ctrl.updateSipTrunking = function(sip_trunking) {
    ctrl.sas.sip_trunking = sip_trunking;
    ctrl.invalid_sip_trunking = true;
    if (!ctrl.sas.sip_trunking){
      ctrl.setSipTrunking();
    }else{
      for (let i = 0; i < ctrl.sas.peers.length; i++) {
        let peer = ctrl.sas.peers[i].name;
        let sip_trunking = ctrl.sas.sip_trunking.substring(1, ctrl.sas.sip_trunking.length - 1)
        if (peer == sip_trunking) {
          ctrl.sas.sip_trunking = ctrl.sas.sip_trunking;
          ctrl.invalid_sip_trunking = false;
          break;
        }
      }
      if (ctrl.invalid_sip_trunking)
        ctrl.setSipTrunking();
    }
  };

  /**
   * Set automatically first peer value to SIP trunking.
   * @function setSipTrunking
   * @memberof SASController
   */
  ctrl.setSipTrunking = function() {
    if (ctrl.sas.peers[0]){
      ctrl.sas.sip_trunking = " " + ctrl.sas.peers[0].name + " ";
      ctrl.invalid_sip_trunking = false;
    }
  };

  /**
   * Sort extensions in ascending values.
   * @function extenSort
   * @memberof SASController
   */
  ctrl.extenSort = function() {
    ctrl.sas.extens.sort(function (a, b) {
      if (a.name > b.name) {
        return 1;
      }
      if (a.name < b.name) {
        return -1;
      }
      return 0;
    });
  };

  /*
   * FileUploader Callback.
   * It is activated when the upload of an item is completed.
   */
  ctrl.uploader.onCompleteItem = function(fileItem, response, status, headers) {
    if (status == 201) {
      ctrl.importConfig();
    }
  };

  ctrl.uploader.onAfterAddingFile = function(item) {
    if (item._file.size/1024/1024 >= 1){
      ctrl.alert_import = 'invalid size';
    }else{
      ctrl.upload_filename = item._file.name;
      ctrl.alert_import = false;
    }
  };

  /**
   * Call the FileUploader object to upload file.
   * @function configImport
   * @memberof SASController
   */
  ctrl.applyImport = function() {
    ctrl.importing = true;
    ctrl.uploader.uploadAll();
  };

  /**
   * Import config file after upload.
   */
  ctrl.importConfig = function() {
    // API PUT request
    $http({
      method: 'PUT',
      url: '/api/config/import_sas',
      data: { 'filename' : ctrl.upload_filename },
      }).then(function(response) {
        // First function handles success
        ctrl.importing = false;
        ctrl.alert_import = 'success';
      }, function(response) {
        // Second function handles error
        ctrl.importing = false;
        if (response.status < 500)
          ctrl.alert_import = response.data;
        else
          ctrl.alert_import = 'unknown';
      });
      ctrl.get();
  };

  /**
   * Check if value to 'Outgoing CallerID' is correct.
   * If isn't correct then print it in web page.
   * We can accept a empty string, but the default
   * will '{}', the API will do the default.
   *
   * @function validateOutgoingCalled
   * @memberof DialplanController
   */
   ctrl.validateOutgoingCalled = function(rule) {
    var regex = /^\+?([\d#*{}:\-])*$/;
    rule.invalid_outgoing_called_pattern = !regex.test(rule.outgoing_called);
    ctrl.checkInvalidConfig();
  };

  /**
   * Check if value to 'Outgoing CallerID' is correct.
   * If isn't correct then print it in web page.
   * We can accept a empty string, but the default
   * will '{}', the API will do the default.
   *
   * @function validateOutgoingCallerID
   * @memberof DialplanController
   */
   ctrl.validateOutgoingCallerID = function(rule) {
    var regex = /^\+?([\d#*{}:\-])*$/;
    rule.invalid_outgoing_callerid_pattern = !regex.test(rule.outgoing_callerid);
    ctrl.checkInvalidConfig();
  };

  /**
   * When we add a new rule it's necessary that the user
   * attach information about the rule name.
   * We don't accept.
   *
   * @function validateRuleName
   * @memberof DialplanController
   */
   ctrl.validateRuleName = function(rule) {
    regex = /^([\w])+$/;

    rule.invalid_rule_name = false;
    if (!regex.test(rule.name)) {
      /* Invalid rule name */
      rule.invalid_rule_name = true;
    } else {
      // Check if this name already exists in another rule
      var count = 0;
      for (var i = 0; i < ctrl.rules.length; i++) {
        if (ctrl.rules[i].name == rule.name)
          count++;
      }

      if (count > 1) {
        rule.invalid_rule_name = true;
      }
    }
    ctrl.checkInvalidConfig();
  };

  /**
   * Check if value to 'Answer Timeout' is correct.
   * Accept only numbers, we don't have a maximum value.
   * We don't accept a empty string.
   * If isn't correct then print it in web page.
   *
   * @function validateAnswerTimeout
   * @memberof SasController
   */
   ctrl.validateAnswerTimeout = function(rule) {
    regex = /^[0-9]+([0-9]+)?$/;

    rule.invalid_answer_timeout = !regex.test(rule.answer_timeout);
    ctrl.checkInvalidConfig();
  };

  /**
   * Check if value to 'Source' and 'Destination' are not both 'tdm'.
   *
   * @function validateSourDest
   * @memberof SasController
   */
   ctrl.validateSourDest = function(rule) {
    rule.invalid_sour_dest = false;
    if (rule.source == 'tdm' && rule.destination == 'tdm')
      rule.invalid_sour_dest = true;
    ctrl.checkInvalidConfig();
  };

}]);
